--
-- buyableObjects
--
-- @author:    	Xentro (www.fs-uk.com)(Marcus@Xentro.se)
-- @version:    v1.0
-- @date:       2014-04-02
-- 

buyableObjects = {};

buyableObjects.path = g_currentModDirectory;

local buyableObjects_mt = Class(buyableObjects, Object);
InitObjectClass(buyableObjects, "buyableObjects");

function buyableObjects.onCreate(id)
    local instance = buyableObjects:new(g_server ~= nil, g_client ~= nil);
	if instance:load(id) then
		g_currentMission:addOnCreateLoadedObject(instance);
		instance:register(true);
	else
		instance:delete();
	end;
end;
g_onCreateUtil.addOnCreateFunction("buyableObjects", buyableObjects.onCreate);

function buyableObjects:new(isServer, isClient)
    local self = Object:new(isServer, isClient, buyableObjects_mt);
    self.className = "buyableObjects";
	
	self.triggerId = 0;
	self.numObjects = 0;
	self.spawnLocations = {};
	
	self.disableProduction = false;
	
    return self;
end;

function buyableObjects:load(id)
	self.isEnabled = true;
	
	local idName = getUserAttribute(id, "id");
	if idName == nil then
		idName = tostring(id);
	end;
	self.idName = idName;
	
	self.triggerId = id;
	self.spawnTime = Utils.getNoNil(getUserAttribute(id, "spawnTime"), 5000)
	self.buyScale = Utils.getNoNil(getUserAttribute(id, "buyScale"), 1)
	self.heightOffset = Utils.getNoNil(getUserAttribute(id, "heightOffset"), 1);
	self.objectFilename = getUserAttribute(id, "filename");
	
	self.isPallet = Utils.getNoNil(getUserAttribute(id, "isPallet"), false);
	
	if self.objectFilename == nil or self.objectFilename == "" then
		print("buyableObjects - Error: invalid filename!");
		return false;
	else
		self.objectFilename = Utils.getFilename(self.objectFilename, buyableObjects.path);
	end;
	
	if self.isServer then
		addTrigger(self.triggerId, "triggerCallback", self);
	end;
	
	local startHour = getUserAttribute(id, "startHour");
	local stopHour = getUserAttribute(id, "stopHour");
	if startHour ~= nil and startHour ~= 0 and stopHour ~= nil and stopHour ~= 0 then
		self.startHour = startHour;
		self.stopHour = stopHour;
	end;
	
	if g_currentMission.buyableObjects == nil then
		g_currentMission.buyableObjects = {};
	end;
	
	g_currentMission.buyableObjects[self.idName] = self;
	
	return true;
end;

function buyableObjects:delete()
	if self.triggerId ~= 0 and self.isServer then
		removeTrigger(self.triggerId);
	end;
	
	g_currentMission.buyableObjects[self.idName] = nil;
end;

function buyableObjects:readStream(streamId, connection)
end;

function buyableObjects:writeStream(streamId, connection)
end;

function buyableObjects:update(dt)
end;

function buyableObjects:updateTick(dt)
	if self.isServer then
		if self.startHour ~= nil and self.stopHour ~= nil then
			local currentHour = math.floor(g_currentMission.environment.dayTime / 3600000);
			
			if currentHour >= self.stopHour then
				self.disableProduction = true;
			elseif currentHour >= self.startHour then
				self.disableProduction = false;
			end;
		end;
		
		if self.numObjects == 0 and not self.disableProduction then
			if self.nextSpawnTime == nil then
				self.nextSpawnTime = g_currentMission.time + self.spawnTime;
			end;
			
			if g_currentMission.time > self.nextSpawnTime then
				local x, y, z = getWorldTranslation(self.triggerId);
				local rx, ry, rz = getWorldRotation(self.triggerId);
				local object = Bale:new(self.isServer, self.isClient);
				
				if self.isPallet then
					object = FillablePallet:new(self.isServer, self.isClient);
					object:load(self.objectFilename, x, y + self.heightOffset, z, rx, ry, rz);
					object:register();
					object:setFillLevel(object:getCapacity());
				else
					object = Bale:new(self.isServer, self.isClient);
					object:load(self.objectFilename, x, y + self.heightOffset, z, rx, ry, rz);
					object:register();
				end;
				
				self.currentObject = object;
				self.nextSpawnTime = nil;
			end;
		else
			self.nextSpawnTime = nil;
		end;
	end;
end;

function buyableObjects:triggerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
    assert(self.isServer);
	
    if self.isEnabled then
		if otherActorId ~= 0 then
			if onEnter then
				self.numObjects = self.numObjects + 1;
			else
				self.numObjects = math.max(self.numObjects - 1, 0);
			end;
			
			local object = g_currentMission:getNodeObject(otherShapeId);
			
			if object ~= nil then
				if not self.firstRun and self.currentObject == nil then
					if object:isa(Bale) then
						self.currentObject = object;
					end;
					
					self.firstRun = true;
				end;
				
				if self.currentObject ~= nil and object.getValue ~= nil then
					if onLeave then
						if object == self.currentObject then
							g_currentMission:addSharedMoney(-self:getCost(), "other");
							playSample(g_currentMission.cashRegistrySound, 1, 1, 0);
							
							self.currentObject = nil;
						end;
					end;
				end;
			end;
		end;
    end;
end;

function buyableObjects:getCost()
	if self.currentObject ~= nil then
		local difficultyMultiplier = math.max(2 * (3 - g_currentMission.missionStats.difficulty), 1);
		return (self.currentObject:getValue() * self.buyScale) * difficultyMultiplier;
	end;
	
	return 0;
end;